package ru.dbotthepony.mc.otm.entity

import net.minecraft.sounds.SoundEvent
import net.minecraft.sounds.SoundEvents
import net.minecraft.world.InteractionHand
import net.minecraft.world.InteractionResult
import net.minecraft.world.entity.AnimationState
import net.minecraft.world.entity.EntityType
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.ai.attributes.AttributeSupplier
import net.minecraft.world.entity.ai.attributes.Attributes
import net.minecraft.world.entity.ai.goal.RandomLookAroundGoal
import net.minecraft.world.entity.ai.goal.WaterAvoidingRandomStrollGoal
import net.minecraft.world.entity.ai.navigation.GroundPathNavigation
import net.minecraft.world.entity.ai.navigation.PathNavigation
import net.minecraft.world.entity.monster.Monster
import net.minecraft.world.entity.player.Player
import net.minecraft.world.level.Level

class Cleaner(type: EntityType<Cleaner>, level: Level) : Monster(type, level) {
	val idleState = AnimationState()
	init {
		idleState.start(tickCount)
	}

	override fun registerGoals() {
		goalSelector.addGoal(8, RandomLookAroundGoal(this))
		goalSelector.addGoal(7, WaterAvoidingRandomStrollGoal(this, 0.8))
	}

	override fun createNavigation(level: Level): PathNavigation = GroundPathNavigation(this, level)

	override fun isPushable(): Boolean {
		return false
	}

	override fun getControllingPassenger(): LivingEntity? {
		return null
	}

	override fun isMultipartEntity(): Boolean {
		return false
	}

	override fun getHurtSound(damageSource: net.minecraft.world.damagesource.DamageSource): SoundEvent {
		return SoundEvents.HEAVY_CORE_BREAK
	}

	override fun getDeathSound(): SoundEvent {
		return SoundEvents.VAULT_BREAK
	}

	override fun mobInteract(player: Player, hand: InteractionHand): InteractionResult {
		if (!this.level().isClientSide) {
			player.startRiding(this)
			return InteractionResult.SUCCESS
		}
		return InteractionResult.CONSUME
	}

	companion object {
		fun createAttributes() : AttributeSupplier.Builder {
			return createMonsterAttributes()
				.add(Attributes.MAX_HEALTH, 4.0)
				.add(Attributes.MOVEMENT_SPEED, 0.2)
		}
	}
}
